'***************************************************************************
'  This module is presented here only to serve as a sample for
'  ATMEGA2560 program development. BiPOM Electronics provides this module as is and
'  does not guarantee its functionality or suitability for a particular purpose.
'  Please e-mail any questions and suggestions to info@bipom.com
'
'    Module: CB-1XBEE.C
'
'    Revision: 1.01
'
'    Description: The example allows to establish a transparent bridge
'                 between UART0 ( 115200 bps) and RF interface (9600 bps)
'
'     SPI_INTERFACE1,CHANNEL A.
'     JP7,JP4 should be closed.
'     All other jumpers should be open.
'
'     Target board         : MINI-MAX/AVR-C
'     Target processor     : ATMEGA2560
'     CPU Clock            : 14.7456 MHz
'
'    This program is written using BASCOM-AVR Compiler from MCS Elecronics
'
'*****************************************************************************
$regfile = "m2560def.dat"
$crystal = 14745600
$baud = 115200
'
'Config the SPI in master mode.The clock must be a quarter of the slave cpu
Config Spi = Hard , Interrupt = Off , Data Order = Msb , Master = Yes , Polarity = Low , Phase = 0 , Clockrate = 128 , Noss = 1
'
Dim Spi_status As Byte
Dim Spi_value As Byte
Dim Sc16is762_channel As Byte
Dim Xbee_byte As Byte
Dim Uart_byte As Byte
'
Declare Sub Spi_init_sc16is762(byval Baudrate As Long)
Declare Sub Spi_write_sc16is762(byval Reg_address As Byte , Byval Reg_value As Byte)
Declare Sub Spi_read_sc16is762(byval Reg_address As Byte )
Declare Sub Spi_change_output_sc16is762(byval Data_bit As Byte , Byval Output_level As Byte)
Declare Sub Spi_change_rts_sc16is762(byval Output_level As Byte)
Declare Sub Spi_put_sc16is762(byval Xbee_value As Byte)
Declare Sub Spi_get_sc16is762()
Declare Sub Is_xbee_byte_waiting()
'
Const Sc16is762_rhr = &H00                                  ' Receive Holding Register
Const Sc16is762_thr = &H00                                  ' Transmit Holding Register
Const Sc16is762_ier = &H08                                  ' Interrupt Enable Register
Const Sc16is762_fcr = &H10                                  ' FIFO Control Register
Const Sc16is762_iir = &H10                                  ' Interrupt Identification Register
Const Sc16is762_lcr = &H18                                  ' Line Control Register
Const Sc16is762_mcr = &H20                                  ' Modem Control Register
Const Sc16is762_lsr = &H28                                  ' Line Status Register
Const Sc16is762_msr = &H30                                  ' Modem Status Register
Const Sc16is762_spr = &H38                                  ' Scratchpad Register
Const Sc16is762_tcr = &H30                                  'Transmission Control Register
Const Sc16is762_tlr = &H38                                  ' Trigger Level Register
Const Sc16is762_txlvl = &H40                                ' Transmitter FIFO Level register
Const Sc16is762_rxlvl = &H48                                ' Receiver FIFO Level register
Const Sc16is762_iodir = &H50                                ' Programmable I/O pins Direction register
Const Sc16is762_iostate = &H58                              ' Programmable I/O pins State register
Const Sc16is762_iointena = &H60                             ' I/O Interrupt Enable register
Const Sc16is762_reserved = &H68                             ' --
Const Sc16is762_iocontrol = &H70                            ' I/O Control register
Const Sc16is762_efcr = &H78                                 ' Extra Features Control Register
'
Const Sc16is762_dll = &H00                                  ' Division Low Register
Const Sc16is762_dlh = &H08                                  ' Division High Register
Const Sc16is762_efr = &H10                                  ' Enhanced Features Register
Const Sc16is762_xon1 = &H20                                 ' XON1 Register
Const Sc16is762_xon2 = &H28                                 ' XON2 Register
Const Sc16is762_xoff1 = &30                                 ' XOFF1 Register
Const Sc16is762_xoff2 = &H38                                ' XOFF2 Register
'
'We poll PB5 to see if there is any incoming data byte
Const Int_avr_mask = &H20
Const Sc16is762_selected_channel = 0                        'Channel to connect to Xbee interface
Const Sc16is762_frequency = 14745600
Const Sc16is762_prescaler = 1
Const Sc16is762_divider = 16
Const Xbee_baudrate = 9600
Const Xbee_pulse_delay = 100
'PB0 is  a slave select pin
Ss Alias Portb.0                                            'Slave select
Const Ss_mask = &H01
'
'
Waitms 500                                                  'Allow to power up
Print "Mini-Max/AVR-C example"
Print "UART0<->Xbee bridge"
Print "REV 1.01 06/10/2009"
'Init the spi pins
Set Ss
Ddrb = Ddrb Or Ss_mask                                      'Slave select is output
Portb = Portb Or Int_avr_mask                               'Enable internal pullup

Sc16is762_channel = Sc16is762_selected_channel
Spiinit
'Initialize serial interface of XBee module
Call Spi_init_sc16is762(xbee_baudrate)
'Clear Reset pin
Call Spi_change_output_sc16is762(7 , 0)
Waitms Xbee_pulse_delay
Call Spi_change_rts_sc16is762(1)                            'Clear /RTS
Waitms Xbee_pulse_delay
Call Spi_change_output_sc16is762(6 , 0)                     'Clear DTR
Waitms Xbee_pulse_delay
'Clear Reset pin
Call Spi_change_output_sc16is762(7 , 1)
Waitms Xbee_pulse_delay
'
Do
    'Try to get a data byte from UART0
    Uart_byte = Inkey()
    If Uart_byte > 0 Then
      Call Spi_put_sc16is762(uart_byte)
    End If
    'Try to get a data byte from Xbee
    Call Is_xbee_byte_waiting()
    If Spi_value = 0 Then
      Call Spi_get_sc16is762()
      Printbin Spi_value
    End If
Loop
End
'********************************************************************************************
'  Subroutine:    Spi_init_sc16is762
'
'  Description:   Initilizes SC16IS762
'
'  Input:         Baudrate
'
'  Output:        Nothing
'
'********************************************************************************************
Sub Spi_init_sc16is762(byval Baudrate As Long)
Local Divisor As Long , Highdata As Byte , Lowdata As Byte
'Divisor =(frequency / prescaler) /(divider * Baudrate)
   Divisor = Sc16is762_frequency / Sc16is762_prescaler
   Divisor = Divisor / Sc16is762_divider
   Divisor = Divisor / Baudrate
   Lowdata = Divisor And &HFF
   Rotate Divisor , Right , 8
   Highdata = Divisor And &HFF
   'Enable divisor registers
   Call Spi_read_sc16is762(sc16is762_lcr)
   Spi_value = Spi_value Or &H80
   Call Spi_write_sc16is762(sc16is762_lcr , Spi_value)
   'Set baud rate
   Call Spi_write_sc16is762(sc16is762_dll , Lowdata)
   Call Spi_write_sc16is762(sc16is762_dlh , Highdata)
   'Disable divisor registers, 8 data bits, 1 stop bit, no parity
   Call Spi_write_sc16is762(sc16is762_lcr , &H03)
   'Enable / Reset FIFO
   Call Spi_write_sc16is762(sc16is762_fcr , &HC7)
   'Enable RX interrupts
   Call Spi_write_sc16is762(sc16is762_ier , &H01)
End Sub
'********************************************************************************************
'  Subroutine:    Spi_write_sc16is762
'
'  Description:   Writes a data byte to SC16IS762 register
'
'  Input:         Reg_address - Sc16is762 Register & Channel
'                 Reg_value - a data byte to be written to Sc16is762 Register
'
'  Output:        Nothing
'
'********************************************************************************************
Sub Spi_write_sc16is762(byval Reg_address As Byte , Byval Reg_value As Byte)
   Reset Ss                                                 'Enable SPI
   Reg_address = Reg_address Or Sc16is762_channel           'Add channel
   Spiout Reg_address , 1                                   'Transmit register byte
   Spiout Reg_value , 1                                     'Transmit value byte
   Set Ss                                                   'Disable SPI
End Sub
'********************************************************************************************
'  Subroutine:    Spi_read_sc16is762
'
'  Description:   Reads a data byte from SC16IS762 register
'
'  Input:         Reg_address - Sc16is762 Register & Channel
'
'  Output:        Spi_value - a data byte that is read from Sc16is762 Register
'********************************************************************************************
Sub Spi_read_sc16is762(byval Reg_address As Byte )
   Reset Ss                                                 'Enable SPI
   Reg_address = Reg_address Or Sc16is762_channel           'Add channel
   Reg_address = Reg_address Or &H80                        'Add READ bit
   Spiout Reg_address , 1                                   'Transmit register byte
   Spi_value = Spimove(&Hff)                                'Transmit FFh dummy byte to read the register
   Set Ss                                                   'Disable SPI
End Sub
'********************************************************************************************
'  Subroutine:    Spi_change_output_sc16is762
'
'  Description:   Configures GPIO as output and sets/clears the port pin
'
'  Input:         Data_bit - data bit to be changed
'                 Output_level - output level (0,1)
'
'  Output:        Nothing
'
'********************************************************************************************
Sub Spi_change_output_sc16is762(byval Data_bit As Byte , Byval Output_level As Byte)
Local Output_mask As Byte , Output_inv_mask As Byte , State_value As Byte
   Output_mask = 1
   Rotate Output_mask , Left , Data_bit
   Output_inv_mask = Output_mask Xor &HFF
   'Read IOSTATE
   Call Spi_read_sc16is762(sc16is762_iostate)
   'Change the desired bit only
   If Output_level = 1 Then
      State_value = Spi_value Or Output_mask                'Set the desired bit
   Else
      State_value = Spi_value And Output_inv_mask           'Clear the desired bit
   End If
   'Write IOSTATE
   Call Spi_write_sc16is762(sc16is762_iostate , State_value)
   'Read IODIR
   Call Spi_read_sc16is762(sc16is762_iodir)
   'Change the desired bit only
   Spi_value = Spi_value Or Output_mask                     'Set the desired bit
   'Write IODIR
   Call Spi_write_sc16is762(sc16is762_iodir , Spi_value)
   'Write IOSTATE  again
   Call Spi_write_sc16is762(sc16is762_iostate , State_value)
End Sub
'********************************************************************************************
'  Subroutine:    Spi_change_rts_sc16is762
'
'  Description:   Sets/clears the RTS  pin
'
'  Input:         Output_level - output level (0,1)
'
'  Output:        Nothing
'
'********************************************************************************************
Sub Spi_change_rts_sc16is762(byval Output_level As Byte)
   'Read MCR
   Call Spi_read_sc16is762(sc16is762_mcr)
   'Change the desired bit only
   If Output_level = 1 Then
      Spi_value = Spi_value Or &H02                         'Set the desired bit
   Else
      Spi_value = Spi_value And &HFD                        'Clear the desired bit
   End If
   'Write MCR
   Call Spi_write_sc16is762(sc16is762_mcr , Spi_value)
End Sub
'********************************************************************************************
'  Subroutine:    Spi_put_sc16is762
'
'  Description:   Puts a data byte to Xbee
'
'  Input:         Xbee_value - a data byte to be sent over RF link
'
'  Output:        Nothing
'
'********************************************************************************************
Sub Spi_put_sc16is762(byval Xbee_value As Byte)
   'Send a data byte
   Call Spi_write_sc16is762(sc16is762_thr , Xbee_value)
   'Wait for THR empty
   Do
      'Read LSR register to check if THR is empty
      Call Spi_read_sc16is762(sc16is762_lsr)
      Spi_value = Spi_value And &H20
      If Spi_value = &H20 Then Exit Do
   Loop
End Sub
'********************************************************************************************
'  Subroutine:    Spi_get_sc16is762
'
'  Description:   Gets a data byte  from Xbee
'
'  Input:         Nothing
'
'  Output:        Spi_value - a data byte that is read from
'
'********************************************************************************************
Sub Spi_get_sc16is762(byval Xbee_value As Byte)
Local Iir_status As Byte
   'Read IIR register to identify interrupts
   Call Spi_read_sc16is762(sc16is762_iir)
   Iir_status = Spi_value And &H04
   'Read RHR register
   Call Spi_read_sc16is762(sc16is762_rhr)
   If Iir_status = 0 Then
      Spi_value = &H3F                                      ' send ?
   End If
End Sub
'********************************************************************************************
'  Subroutine:    Is_Xbee_byte_waiting
'
'  Description:   Checks if there is  Xbee  pending byte
'
'  Input:         Nothing
'
'  Output:        Spi_value = 0, there is a pending data byte
'                 Spi_value = Int_Avr_mask, no pending data byte
'
'********************************************************************************************
Sub Is_xbee_byte_waiting()
   Spi_value = Pinb And Int_avr_mask
End Sub
'EOF
